__version__ = "1.0.0"

# Hooray for uvloop!
from . import _helpers
from .core import Config, Session, SynchronizedSession
from .primitives import Asset, AssetFile, AssetType, LicenseType


__all__ = [
    "Asset", "AssetFile", "AssetType", "LicenseType",
    "Config", "Session", "SynchronizedSession",
]



import asyncio
import functools
import logging
from typing import AsyncGenerator, Generator, List, Optional, TypeVar


logger = logging.getLogger(__name__)

T = TypeVar("T")

__all__ = ["block_on", "collect", "synchronize_generator"]


class _AsyncProxy:
    def __init__(self, __proxy, __loop: asyncio.BaseEventLoop):
        self.__proxy = __proxy
        self.__loop = __loop

    def __getattr__(self, func_name):
        func = getattr(self.__proxy, func_name)

        @functools.wraps(func)
        def call(*args, **kwargs):
            task = func(*args, **kwargs)
            return self.__loop.run_until_complete(task)
        return call


def block_on(obj, loop=None):
    if loop is None:
        loop = obj.loop
    return _AsyncProxy(obj, loop)


async def collect(generator: AsyncGenerator[T, None]) -> List[T]:
    results = []
    async for x in generator:
        results.append(x)
    return results


def synchronize_generator(
        async_gen: AsyncGenerator[T, None],
        loop: Optional[asyncio.BaseEventLoop]=None) -> Generator[T, None, None]:
    """
    Maps ``__anext__`` to ``__next__`` to synchronously interact with async generators.

    If you would write the following async code:

    .. code-block: python

        async for x in my_gen:
            print(x)

    Then you can synchronize this call with the following:

    .. code-block: python

        for x in synchronize_generator(my_gen):
            print(x)

    :param async_gen: An asynchronous generator you'd like to synchronously iterate
    :param loop: The event loop to use; probably the same one that created your async generator
    :return: An object that
    """
    if loop is None:
        loop = asyncio.get_event_loop()

    while True:
        try:
            yield loop.run_until_complete(async_gen.__anext__())
        except StopAsyncIteration:
            break


def _install_uvloop():
    try:
        import asyncio
        import uvloop
        asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
        logger.debug("installed uvloop.EventLoopPolicy")
    except ImportError:
        pass


def enable_speedups():
    _install_uvloop()


enable_speedups()



import enum
import json
from typing import List, NamedTuple


__all__ = ["Asset", "AssetFile", "AssetType", "LicenseType"]


class AssetType(enum.Enum):
    ART_2D = "2D Art"
    ART_3D = "3D Art"
    CONCEPT_ART = "Concept Art"
    TEXTURE = "Texture"
    MUSIC = "Music"
    SOUND_EFFECT = "Sound Effect"
    DOCUMENT = "Document"


class LicenseType(enum.Enum):
    CC_BY_40 = "CC-BY 4.0"
    CC_BY_30 = "CC-BY 3.0"
    CC_BY_SA_40 = "CC-BY-SA 4.0"
    CC_BY_SA_30 = "CC-BY-SA 3.0"
    GPL_30 = "GPL 3.0"
    GPL_20 = "GPL 2.0"
    OGA_BY_30 = "OGA-BY 3.0"
    CC0 = "CC0"
    LGPL_30 = "LGPL 3.0"
    LGPL_21 = "LGPL 2.1"


class AssetFile(NamedTuple):
    id: str
    etag: str
    size: int

    def to_json(self) -> dict:
        return {
            "id": self.id,
            "etag": self.etag,
            "size": self.size}


class Asset:
    id: str
    author: str
    type: AssetType
    licenses: List[LicenseType]
    tags: List[str]
    favorites: int
    files: List[AssetFile]

    def __init__(self, id, author, type, licenses, tags, favorites, files) -> None:
        self.id = id
        self.author = author
        self.type = type
        self.licenses = licenses
        self.tags = tags
        self.favorites = favorites
        self.files = files

    def to_json(self) -> dict:
        return {
            "id": self.id,
            "author": self.author,
            "type": self.type.value,
            "licenses": [license.value for license in self.licenses],
            "tags": self.tags,
            "favorites": self.favorites,
            "files": [file.to_json() for file in self.files]}

    def __repr__(self) -> str:
        return json.dumps(self.to_json(), sort_keys=True, indent=4)



# Core operations for downloading, searching on OpenGameArt.org
import asyncio
import json
import pathlib
import urllib.parse
from configparser import ConfigParser
from typing import AsyncGenerator, Dict, Generator, List, Optional

import aiohttp

from ._helpers import synchronize_generator
from .parsing import (
    Translations,
    parse_asset,
    parse_last_search_page,
    parse_search_results,
)
from .primitives import Asset, AssetFile, AssetType, LicenseType


__all__ = ["Config", "Session", "SynchronizedSession"]

DEFAULT_CONFIG_LOCATION = "~/.oga/config"
CONFIG_SECTION_NAME = "oga"


class Config:
    url: str
    max_conns: int
    root_dir: str

    def __init__(self, *, url: str, max_conns: int, root_dir: str) -> None:
        self.url = url
        self.max_conns = max_conns
        self.root_dir = root_dir

    @classmethod
    def default(cls):
        return Config(
            url="https://opengameart.org",
            max_conns=5,
            root_dir="~/.oga")

    @classmethod
    def from_file(cls, file_path: Optional[str]=None) -> Optional["Config"]:
        file_path = file_path or DEFAULT_CONFIG_LOCATION
        try:
            filename = pathlib.Path(file_path).expanduser().resolve()
        except FileNotFoundError:
            return None
        default = Config.default()
        parser = ConfigParser()
        parser.read(filename)
        try:
            section = parser[CONFIG_SECTION_NAME]
        except KeyError:
            return default
        return Config(
            url=section.get("url", fallback=default.url),
            max_conns=section.getint("max_conns", fallback=default.max_conns),
            root_dir=section.get("root_dir", fallback=default.root_dir))


async def search(session: aiohttp.ClientSession, base_query: str) -> AsyncGenerator[str, None]:
    page = 0

    async def fetch() -> bytes:
        url = f"{base_query}&page={page}"
        async with session.get(url) as response:
            return await response.read()

    # Special case for first page since we may not continue
    data = await fetch()
    last_page = parse_last_search_page(data)
    asset_ids = parse_search_results(data)
    for asset_id in asset_ids:
        yield asset_id

    while page < last_page:
        page += 1
        data = await fetch()
        asset_ids = parse_search_results(data)
        for asset_id in asset_ids:
            yield asset_id


class Session:
    """
    Provides asynchronous methods for querying and fetching
    (against a cache manifest) individual assets and collections.

    Usage::

        >>> session = Session()
        # easier blocking calls
        >>> run = session.loop.run_until_complete

        >>> asset = run(session.describe_asset("imminent-threat"))
        >>> run(session.download_asset(asset))
        # cache hit, returns immediately
        >>> run(session.download_asset(asset))

    """
    def __init__(self, config: Optional[Config]=None, loop: Optional[asyncio.AbstractEventLoop]=None) -> None:
        if config is None:
            config = Config.default()
        if loop is None:
            loop = asyncio.new_event_loop()
        self.loop = loop
        self.config = config

        # Set up connection limiting according to config
        conn = aiohttp.TCPConnector(limit=config.max_conns, limit_per_host=config.max_conns, loop=loop)
        self._session = aiohttp.ClientSession(connector=conn, loop=loop)
        self._file_manager = LocalFileManager(config)

    def __del__(self) -> None:
        self._session.close()

    def search(
            self, *,
            keys: Optional[str]=None,
            title: Optional[str]=None,
            submitter: Optional[str]=None,
            sort_by: Optional[str]=None,
            descending: Optional[bool]=None,
            types: Optional[List[AssetType]]=None,
            licenses: Optional[List[LicenseType]]=None,
            tags: Optional[List[str]]=None,
            tag_operation: Optional[str]=None) -> AsyncGenerator[str, None]:
        """
        :param keys: appears to search the entire page (including comments, tags...)
        :param title: appears in the asset title
        :param submitter: part or all of the submitter's username.  Not always the author (eg. "submitted by")
        :param sort_by: "favorites", "created", "views"
        :param descending: True if sorting descending, False for ascending.
        :param types: allowed asset types.  Leave blank to allow all.
        :param licenses: allowed licenses.  Leave blank to allow all.
        :param tags: List of tags to match or avoid, depending on ``tag_operation``
        :param tag_operation: "or", "and", "not", "empty", "not empty"
        """
        # 0) Apply defaults
        keys = keys or ""
        title = title or ""
        submitter = submitter or ""
        sort_by = (sort_by or "favorites").lower()
        descending = True if descending is None else descending
        types = types or []
        licenses = licenses or []
        tags = tags or []
        tag_operation = (tag_operation or "or").lower()

        # 1) Validate enums
        def validate(param, param_name, allowed):
            if param not in allowed:
                raise ValueError(f"{param_name} must be one of {allowed} but was {param!r}")

        validate(sort_by, "sort_by", {"favorites", "created", "views"})
        validate(tag_operation, "tag_operation", {"or", "and", "not", "empty", "not empty"})

        # 2) transform params into request format
        sort_by = {
            "favorites": "count",
            "created": "created",
            "views": "totalcount",
        }[sort_by]
        sort_order = "DESC" if descending else "ASC"
        types = [Translations.asset_type_search_values[x] for x in types]
        licenses = [Translations.license_search_values[x] for x in licenses]
        url = f"{self.config.url}/art-search-advanced?"

        # 3) build values into url
        quote = urllib.parse.quote_plus
        base_query = "&".join([
            url,
            f"keys={quote(keys)}",
            f"title={quote(title)}",
            f"field_art_tags_tid_op={quote(tag_operation)}",
            f"field_art_tags_tid={quote(','.join(tags))}",
            f"name={quote(submitter)}",
            f"sort_by={sort_by}",
            f"sort_order={sort_order}",
            f"items_per_page=144"
        ])
        if types:
            type_query = "&".join([
                f"field_art_type_tid%5B%5D={type}"
                for type in types
            ])
            base_query += "&" + type_query
        if licenses:
            license_query = "&".join([
                f"field_art_licenses_tid%5B%5D={license}"
                for license in licenses
            ])
            base_query += "&" + license_query
        return search(self._session, base_query)

    async def describe_asset(self, asset_id: str) -> Asset:
        url = f"{self.config.url}/content/{asset_id}"
        async with self._session.get(url) as response:
            partial_asset = parse_asset(asset_id, await response.read())
        tasks = [
            self.describe_asset_file(asset_file_id)
            for asset_file_id in partial_asset["files"]]
        if tasks:
            tasks, _ = await asyncio.wait(
                tasks, loop=self.loop, return_when=asyncio.ALL_COMPLETED)
            partial_asset["files"] = [task.result() for task in tasks]
        return Asset(**partial_asset)

    async def describe_asset_file(self, asset_file_id: str) -> AssetFile:
        url = f"{self.config.url}/sites/default/files/{asset_file_id}"
        async with self._session.head(url) as response:
            headers = response.headers
        etag = headers["ETag"]  # type: str
        if etag.startswith("\"") and etag.endswith("\""):
            etag = etag[1:-1]
        return AssetFile(
            id=asset_file_id,
            etag=etag,
            size=int(headers["Content-Length"]))

    async def download_asset(self, asset: Asset) -> None:
        if not asset.files:
            return
        tasks = [
            self.download_asset_file(asset.id, asset_file)
            for asset_file in asset.files]
        await asyncio.wait(tasks, loop=self.loop, return_when=asyncio.ALL_COMPLETED)

    async def download_asset_file(self, asset_id: str, asset_file: AssetFile) -> None:
        current_etag = self._file_manager.get_etag(asset_id=asset_id, asset_file_id=asset_file.id)
        # cache hit
        if current_etag and current_etag == asset_file.etag:
            return

        url = f"{self.config.url}/sites/default/files/{asset_file.id}"
        async with self._session.get(url) as response:
            data = await response.read()
            self._file_manager.save(
                asset_id=asset_id,
                asset_file_id=asset_file.id,
                etag=asset_file.etag,
                data=data
            )


class LocalFileManager:
    def __init__(self, config: Config) -> None:
        self.config = config
        self._cache = {}  # type: Dict[str, Dict[str, str]]

    def load(self, *, asset_id: str, asset_file_id: str) -> Optional[bytes]:
        asset_file_path = self._path_to_content_dir(asset_id) / asset_file_id
        try:
            return asset_file_path.read_bytes()
        except FileNotFoundError:
            return None

    def save(self, *, asset_id: str, asset_file_id: str, etag: str, data: bytes) -> None:
        asset_file_path = self._path_to_content_dir(asset_id) / asset_file_id
        asset_file_path.write_bytes(data)
        self._set_etag(asset_id=asset_id, asset_file_id=asset_file_id, etag=etag)

    def delete(self, *, asset_id: str, asset_file_id: str) -> None:
        asset_file_path = self._path_to_content_dir(asset_id) / asset_file_id
        asset_file_path.unlink()
        self._clear_etag(asset_id=asset_id, asset_file_id=asset_file_id)

    def get_etag(self, *, asset_id: str, asset_file_id: str) -> Optional[str]:
        """
        Also ensures asset file exists locally.
        File checksums aren't validated yet, because OGA doesn't publish them.
        """
        self._load_cache(asset_id=asset_id, force=False)
        last_etag = self._cache[asset_id].get(asset_file_id, None)
        asset_file_path = self._path_to_content_dir(asset_id) / asset_file_id
        if asset_file_path.exists():
            return last_etag
        # file doesn't exist but cache is stale
        if last_etag:
            self._clear_etag(asset_id=asset_id, asset_file_id=asset_file_id)
        return None

    def _set_etag(self, *, asset_id: str, asset_file_id: str, etag: str) -> None:
        self._load_cache(asset_id=asset_id, force=True)
        self._cache[asset_id][asset_file_id] = etag
        self._save_cache(asset_id=asset_id)

    def _clear_etag(self, *, asset_id: str, asset_file_id: str) -> None:
        self._load_cache(asset_id=asset_id, force=True)
        self._cache[asset_id][asset_file_id] = None
        self._save_cache(asset_id=asset_id)

    def _load_cache(self, *, asset_id: str, force: bool=False) -> None:
        if asset_id in self._cache and not force:
            return
        cache_file = self._path_to_cache(asset_id)
        try:
            data = json.loads(cache_file.read_text())
        except (FileNotFoundError, json.JSONDecodeError):
            cache_file.write_text("{}")
            data = {}
        self._cache[asset_id] = data

    def _save_cache(self, *, asset_id: str) -> None:
        self._load_cache(asset_id=asset_id, force=False)
        cache_file = self._path_to_cache(asset_id)
        cache_file.write_text(json.dumps(self._cache[asset_id], sort_keys=True, indent=4))

    def _path_to_cache(self, asset_id: str) -> pathlib.Path:
        path = (pathlib.Path(self.config.root_dir) / "cache" / asset_id).expanduser()
        path.parent.mkdir(parents=True, exist_ok=True)
        return path

    def _path_to_content_dir(self, asset_id: str) -> pathlib.Path:
        path = (pathlib.Path(self.config.root_dir) / "assets" / asset_id).expanduser()
        path.mkdir(parents=True, exist_ok=True)
        return path


class SynchronizedSession:
    def __init__(self, session: Optional[Session]=None):
        if session is None:
            session = Session()
        self._session = session

    def search(
            self, *,
            keys: Optional[str]=None,
            title: Optional[str]=None,
            submitter: Optional[str]=None,
            sort_by: Optional[str]=None,
            descending: Optional[bool] = None,
            types: Optional[List[AssetType]]=None,
            licenses: Optional[List[LicenseType]]=None,
            tags: Optional[List[str]]=None,
            tag_operation: Optional[str]=None) -> Generator[str, None, None]:
        """
        :param keys: appears to search the entire page (including comments, tags...)
        :param title: appears in the asset title
        :param submitter: part or all of the submitter's username.  Not always the author (eg. "submitted by")
        :param sort_by: "favorites", "created", "views"
        :param descending: True if sorting descending, False for ascending.
        :param types: allowed asset types.  Leave blank to allow all.
        :param licenses: allowed licenses.  Leave blank to allow all.
        :param tags: List of tags to match or avoid, depending on ``tag_operation``
        :param tag_operation: "or", "and", "not", "empty", "not empty"
        """
        loop = self._session.loop
        search_task = self._session.search(
            keys=keys, title=title, submitter=submitter, sort_by=sort_by, descending=descending, types=types,
            licenses=licenses, tags=tags, tag_operation=tag_operation
        )
        return synchronize_generator(search_task, loop=loop)

    def batch_describe_assets(self, asset_ids: List[str]) -> Dict[str, Asset]:
        loop = self._session.loop
        tasks = [self._session.describe_asset(asset_id) for asset_id in asset_ids]
        if not tasks:
            return {}
        wait_for = asyncio.wait(tasks, loop=loop, return_when=asyncio.ALL_COMPLETED)
        done, _ = loop.run_until_complete(wait_for)
        assets = [task.result() for task in done]
        return {asset.id: asset for asset in assets}

    def batch_download_assets(self, assets: List[Asset]) -> None:
        loop = self._session.loop
        tasks = [self._session.download_asset(asset) for asset in assets]
        if not tasks:
            return
        wait_for = asyncio.wait(tasks, loop=loop, return_when=asyncio.ALL_COMPLETED)
        loop.run_until_complete(wait_for)



"""Command line implementation of oga."""
import pathlib
from typing import List, Optional

import click

from oga.core import Config, Session
from oga.primitives import AssetType


def init_config(
        config_path: Optional[str],
        root_dir: Optional[str],
        url: Optional[str],
        max_conns: Optional[int]) -> Config:
    config = Config.from_file(config_path)
    if root_dir is not None:
        config.root_dir = pathlib.Path(root_dir).expanduser()
    if url is not None:
        config.url = url
    if max_conns is not None:
        config.max_conns = max_conns
    return config


cli_type_map = {
    "2d": AssetType.ART_2D,
    "3d": AssetType.ART_3D,
    "concept": AssetType.CONCEPT_ART,
    "texture": AssetType.TEXTURE,
    "music": AssetType.MUSIC,
    "sfx": AssetType.SOUND_EFFECT,
    "doc": AssetType.DOCUMENT,
}
rev_cli_type_map = {v: k for k, v in cli_type_map.items()}


async def cli_describe(session: Session, asset_id: str, verbose: bool) -> str:
    asset = await session.describe_asset(asset_id=asset_id)
    if verbose:
        return str(asset)
    else:
        summary = {
            "id": asset.id,
            "type": rev_cli_type_map[asset.type],
            "favorites": asset.favorites,
            "tags": len(asset.tags)
        }
        template = "{id} {type} ({favorites} favorites, {tags} tags)"
        return template.format(**summary)


@click.group()
@click.option("--config-path", type=click.Path(exists=True, file_okay=True, dir_okay=False), required=False)
@click.option("--root-dir", type=click.Path(exists=False, dir_okay=True, file_okay=False))
@click.option("--url", type=str, required=False)
@click.option("--max-conns", type=int, required=False)
@click.pass_context
def cli(ctx, config_path: Optional[str], root_dir: Optional[str], url: Optional[str], max_conns: Optional[int]):
    """Search and download assets from OpenGameArt.org"""
    config = init_config(config_path, root_dir, url, max_conns)
    session = Session(config)
    ctx.obj = session


@cli.command("describe")
@click.argument("asset")
@click.option("--verbose/--summary", is_flag=True, default=False)
@click.pass_obj
def describe_asset(session: Session, asset: str, verbose: bool):
    """Look up a single ASSET."""
    description = session.loop.run_until_complete(cli_describe(session, asset, verbose))
    print(description, flush=True)


@cli.command("download")
@click.argument("asset")
@click.pass_obj
def download_asset(session: Session, asset: str):
    """Download files for a single ASSET."""
    asset = session.loop.run_until_complete(session.describe_asset(asset_id=asset))
    session.loop.run_until_complete(session.download_asset(asset=asset))


@cli.command("search")
@click.option("--verbose/--summary", is_flag=True, default=False)
@click.option("--keys", help="Search the whole page", type=str, default=None)
@click.option("--title", help="Search the asset title", type=str, default=None)
@click.option("--submitter", help="Search the submitter name", type=str, default=None)
@click.option("--sort-by", type=click.Choice(["favorites", "created", "views"]), default="favorites")
@click.option("--descending/--ascending", help="sort order", is_flag=True, default=True)
@click.option("--type", type=click.Choice(["2d", "3d", "concept", "texture", "music", "sfx", "doc"]), multiple=True)
@click.option("--tag", help="freeform tag", multiple=True, type=str)
@click.option("--tag-op", type=click.Choice(["or", "and", "not", "empty", "not-empty"]), default="or")
@click.pass_obj
def search_assets(
        session: Session, verbose: bool,
        keys: Optional[str], title: Optional[str], submitter: Optional[str],
        sort_by: str, descending: bool, type: List[str], tag: List[str], tag_op: str):
    """Search for an asset."""
    types = [cli_type_map[t] for t in type]
    search = session.search(
        keys=keys,
        title=title,
        submitter=submitter,
        sort_by=sort_by,
        descending=descending,
        types=types,
        tags=tag,
        tag_operation=tag_op
    )

    async def process():
        async for asset_id in search:
            description = await cli_describe(session, asset_id, verbose)
            print(description, flush=True)

    session.loop.run_until_complete(process())



import re
import urllib.parse
from typing import List

import bs4

from .primitives import AssetType, LicenseType


class Translations:
    license_search_values = {
        LicenseType.CC_BY_40: "17981",
        LicenseType.CC_BY_30: "2",
        LicenseType.CC_BY_SA_40: "17982",
        LicenseType.CC_BY_SA_30: "3",
        LicenseType.GPL_30: "6",
        LicenseType.GPL_20: "5",
        LicenseType.OGA_BY_30: "10310",
        LicenseType.CC0: "4",
        LicenseType.LGPL_30: "8",
        LicenseType.LGPL_21: "7",
    }

    asset_type_search_values = {
        AssetType.ART_2D: "9",
        AssetType.ART_3D: "10",
        AssetType.CONCEPT_ART: "7273",
        AssetType.TEXTURE: "14",
        AssetType.MUSIC: "12",
        AssetType.SOUND_EFFECT: "13",
        AssetType.DOCUMENT: "11",
    }


def parse_asset(asset_id: str, data: bytes) -> dict:
    text = data.decode("utf-8")
    soup = bs4.BeautifulSoup(text, "html.parser")

    # 0) author
    authors = soup.find_all(class_="field-name-author-submitter")
    assert len(authors) == 1
    authors = authors[0].find_all("a")
    for maybe_author in authors:
        if maybe_author["href"].startswith("/users/"):
            author = maybe_author["href"][7:]
            break
    else:
        author = None

    # 1) type
    types = soup.find_all(class_="field-name-field-art-type")
    assert len(types) == 1
    type = AssetType(types[0].a.text)

    # 2) licenses
    license_section = soup.find_all(class_="field-name-field-art-licenses")
    assert len(license_section) == 1
    licenses = [
        LicenseType(license.text)
        for license in license_section[0].find_all(class_="license-name")]

    # 3) tags
    tags_section = soup.find_all(class_="field-name-field-art-tags")
    assert len(tags_section) == 1
    tags = [tag.text for tag in tags_section[0].find_all("a")]

    # 4) favorites
    favorites_section = soup.find_all(class_="field-name-favorites")
    assert len(favorites_section) == 1
    favorites = int(favorites_section[0].find(class_="field-item").text)

    # 5) files
    files_section = soup.find_all(class_="field-name-field-art-files")
    assert len(files_section) == 1
    files = []
    for container_el in files_section[0].find_all(class_="file"):
        url = container_el.a["href"]
        file_id = urllib.parse.unquote(url).split("/sites/default/files/")[-1]
        files.append(file_id)
    return {
        "id": asset_id,
        "author": author,
        "type": type,
        "licenses": licenses,
        "tags": tags,
        "favorites": favorites,
        "files": files}


def parse_search_results(data: bytes) -> List[str]:
    text = data.decode("utf-8")
    soup = bs4.BeautifulSoup(text, "html.parser")
    containers = soup.find_all(class_="view-display-id-search_art_advanced")
    asset_ids = []
    if len(containers) == 0:
        return asset_ids
    assert len(containers) == 1
    container = containers[0]
    spans = container.find_all("span", class_="art-preview-title")
    for span in spans:
        url = span.a["href"]  # type: str
        assert url.startswith("/content/")
        asset_ids.append(url[9:])
    return asset_ids


def parse_last_search_page(data: bytes) -> int:
    text = data.decode("utf-8")
    soup = bs4.BeautifulSoup(text, "html.parser")
    pagers = soup.find_all(class_="pager-last")
    if not pagers:
        return 0  # there is only one page, it just happens to be empty
    assert len(pagers) == 1
    pager = pagers[0]
    url = pager.a["href"]
    page_regex = re.compile("&page=(?P<page>[0-9]+)")
    match = page_regex.search(url)
    assert match
    return int(match.groupdict()["page"])



import os
import pathlib
from setuptools import setup, find_packages

HERE = pathlib.Path(os.path.abspath(os.path.dirname(__file__)))
README = (HERE / "README.rst").read_text()
VERSION = "VERSION-NOT-FOUND"
for line in (HERE / "oga" / "__init__.py").read_text().split("\n"):
    if line.startswith("__version__"):
        VERSION = eval(line.split("=")[-1])

REQUIREMENTS = [
    "aiohttp>=2.3",
    "beautifulsoup4>=4.6",
    "click>=6.7"
]

if __name__ == "__main__":
    setup(
        name="oga",
        version=VERSION,
        description="Library for interacting with OpenGameArt.org assets",
        long_description=README,
        author="Joe Cross",
        author_email="joe.mcross@gmail.com",
        url="https://github.com/numberoverzero/oga",
        license="MIT",
        platforms="any",
        include_package_data=True,
        packages=find_packages(exclude=("docs", "examples", "scripts", "tests")),
        install_requires=REQUIREMENTS,
        entry_points={
            "console_scripts": [
                "oga = oga.main.cli:cli"
            ]
        }
    )
